# -*- coding: utf-8 -*-

import hashlib
from base64 import urlsafe_b64encode

from .constants import SIGNATURE_MODE_QUERY

from .compat import iteritems
from .compat import urlparse
from .compat import quote
from .compat import unquote
from .compat import b

from ._version import __version__


class UrlHelper(object):
    def __init__(
            self,
            domain,
            path,
            scheme="https",
            sign_key=None,
            sign_mode=SIGNATURE_MODE_QUERY,
            sign_with_library_version=True,
            opts={}):

        self._scheme = scheme
        self._host = domain
        self._path = path
        self._sign_key = sign_key
        self._sign_with_library_version = sign_with_library_version

        if sign_mode != SIGNATURE_MODE_QUERY:
            raise Exception("Path signatures are not supported yet.")

        self._sign_mode = sign_mode
        self._parameters = {}

        for key, value in iteritems(opts):
            self.set_parameter(key, value)

    @classmethod
    def from_url(cls, url):
        pass

    def set_parameter(self, key, value):
        if value is None or value is False:
            self.delete_parameter(key)
            return

        if isinstance(value, (int, float)):
            value = str(value)

        if key.endswith('64'):
            value = urlsafe_b64encode(value.encode('utf-8'))
            value = value.replace(b('='), b(''))

        self._parameters[key] = value

    def delete_parameter(self, key):
        if key in self._parameters:
            del self._parameters[key]

    def _str_is_ascii(self, s):
        try:
            b(s).decode('ascii')
            return True
        except:
            return False

    def __str__(self):
        query = {}

        for key in self._parameters:
            query[key] = self._parameters[key]

        path = unquote(self._path)

        if self._sign_with_library_version:
            query["ixlib"] = "python-" + __version__

        if path.startswith("http"):
            try:
                path = quote(path, safe="")
            except KeyError:
                path = quote(path.encode('utf-8'), safe="")

        if not path.startswith("/"):
            path = "/" + path  # Fix web proxy style URLs

        if not path.startswith("/http") and not self._str_is_ascii(path):
            try:
                path = quote(path)
            except KeyError:
                path = quote(path.encode('utf-8'))

        query = "&".join(
            (quote(key, "") + "=" + quote(query[key], ""))
            for key in sorted(query))

        if self._sign_key:
            delim = "" if query == "" else "?"
            signing_value = self._sign_key + path + delim + query
            signature = hashlib.md5(signing_value.encode('utf-8')).hexdigest()
            if query:
                query += "&s=" + signature
            else:
                query = "s=" + signature

        return urlparse.urlunparse([
            self._scheme,
            self._host,
            path,
            "",
            query,
            "", ])



# -*- coding: utf-8 -*-

import zlib

from .urlhelper import UrlHelper

from .constants import SHARD_STRATEGY_CYCLE
from .constants import SHARD_STRATEGY_CRC
from .constants import SIGNATURE_MODE_QUERY


class UrlBuilder(object):
    def __init__(
            self,
            domains,
            use_https=True,
            sign_key=None,
            sign_mode=SIGNATURE_MODE_QUERY,
            shard_strategy=SHARD_STRATEGY_CRC,
            sign_with_library_version=True):

        if not isinstance(domains, list):
            domains = [domains]

        self._domains = domains
        self._sign_key = sign_key
        self._sign_mode = sign_mode
        self._use_https = use_https
        self._shard_strategy = shard_strategy
        self._shard_next_index = 0
        self._sign_with_library_version = sign_with_library_version

    def create_url(self, path, opts={}):
        if self._shard_strategy == SHARD_STRATEGY_CRC:
            crc = zlib.crc32(path.encode('utf-8')) & 0xffffffff
            index = crc % len(self._domains)  # Deterministically choose domain
            domain = self._domains[index]

        elif self._shard_strategy == SHARD_STRATEGY_CYCLE:
            domain = self._domains[self._shard_next_index]
            self._shard_next_index = (
                self._shard_next_index + 1) % len(self._domains)

        else:
            domain = self._domains[0]

        scheme = "https" if self._use_https else "http"

        url_obj = UrlHelper(
            domain,
            path,
            scheme,
            sign_key=self._sign_key,
            sign_mode=self._sign_mode,
            sign_with_library_version=self._sign_with_library_version,
            opts=opts)

        return str(url_obj)



from imgix._version import __version__
from setuptools import setup

setup(
    name='imgix',
    version=__version__,
    author='imgix',
    author_email='support@imgix.com',
    packages=['imgix'],
    url='https://github.com/imgix/imgix-python',
    license='MIT',
    description='Python client library for imgix.',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
    ],
    )



# -*- coding: utf-8 -*-

from .urlbuilder import UrlBuilder

from .constants import SHARD_STRATEGY_CYCLE
from .constants import SHARD_STRATEGY_CRC
from .constants import SIGNATURE_MODE_QUERY
from .constants import SIGNATURE_MODE_PATH


__all__ = [
    'UrlBuilder', 'SHARD_STRATEGY_CRC', 'SHARD_STRATEGY_CYCLE',
    'SIGNATURE_MODE_QUERY', 'SIGNATURE_MODE_PATH', '__version__', ]



try:
    # Python 3
    import urllib.parse as urlparse
    from urllib.parse import urlencode
    from urllib.parse import quote
    from urllib.parse import unquote

    def iteritems(a_dict):
        return a_dict.items()

    def b(s):
        return s.encode("latin-1")

except ImportError:
    # Python 2.7
    import urlparse
    from urllib import urlencode
    from urllib import quote
    from urllib import unquote

    def iteritems(a_dict):
        return a_dict.iteritems()

    def b(s):
        return s

__all__ = ['iteritems', 'quote', 'unquote', 'urlparse', 'urlencode']
